/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2012                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/

#include <vcg/complex/complex.h>
#include <vcg/complex/append.h>
#include <wrap/io_trimesh/import_ply.h>
#include <wrap/io_trimesh/export_ply.h>
#include <vcg/complex/algorithms/update/color.h>
using namespace vcg;
using namespace std;


class MyEdge;
class MyFace;
class MyVertex;
struct MyUsedTypes : public vcg::UsedTypes<	vcg::Use<MyVertex>   ::AsVertexType,
                                        vcg::Use<MyEdge>     ::AsEdgeType,
                                        vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex<MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face< MyUsedTypes, vcg::face::FFAdj,  vcg::face::VertexRef, vcg::face::Color4b, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<MyUsedTypes>{};
class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};


int main()
{
  MyMesh m;
  tri::io::ImporterPLY<MyMesh>::Open(m,"2handle.ply");

  printf("Read mesh %i %i\n",m.FN(),m.VN());
  tri::UpdateColor<MyMesh>::PerFaceConstant(m,Color4b::White);
  tri::UpdateTopology<MyMesh>::FaceFace(m);
  tri::UpdateFlags<MyMesh>::FaceClearV(m);
  tri::UpdateFlags<MyMesh>::VertexClearV(m);
  int visitedCnt=0;
  std::vector<MyFace*> faceVec;
  faceVec.push_back(&(m.face[0]));
  faceVec.back()->SetV();
  while(!faceVec.empty())
  {
    int randPos = rand()%faceVec.size();
    std::swap(faceVec[randPos],faceVec.back());
    MyFace *fp=faceVec.back();
    faceVec.pop_back();
    for(int i=0;i<3;i++)
    {
      if(!fp->FFp(i)->IsV())
      {
        MyFace *ffp = fp->FFp(i);
        bool badFace=false;
        int visitedVertexCnt=0;
        for(int j=0;j<3;j++)
        {
          if(ffp->FFp(j)->IsV())
            if(ffp->FFp(j) != fp) badFace =true;
          if(ffp->V(j)->IsV()) visitedVertexCnt++;
        }
        if(badFace==false && visitedVertexCnt<=2)
        {
          faceVec.push_back(ffp);
          ffp->SetV();
          ffp->SetS();
          for(int j=0;j<3;j++)
            ffp->V(j)->SetV();

          ffp->C()=Color4b::Red;
        }
      }
    }
    visitedCnt++;
  }

  MyMesh profile;
  tri::UpdateBounding<MyMesh>::Box(m);
  tri::UpdateNormal<MyMesh>::PerVertexAngleWeighted(m);
  tri::UpdateNormal<MyMesh>::NormalizePerVertex(m);

  float scaleFac=profile.bbox.Diag()/100.0;

  tri::Append<MyMesh,MyMesh>::Selected(profile,m);

  for(int i=0;i<profile.VN();++i)
  {
    profile.vert[i].P()+=profile.vert[i].N()*scaleFac;
  }

  MyMesh::FaceIterator fi;
  for(fi = m.face.begin();fi!= m.face.end();++fi)
  {
    if(fi->IsS())
      tri::Allocator<MyMesh>::DeleteFace(m,*fi);
  }

  /*
  WRONG WAY OF ITERATING: FN() become smaller after each DeleteFace and the last part of the vector is never reached...
  ---> for(int i=0;i<m.FN();++i) <---
  {
    if(fi->IsS())
      tri::Allocator<MyMesh>::DeleteFace(m,m.face[i]);
  }
  */

  printf("Visited %i faces\n",visitedCnt);
  tri::io::ExporterPLY<MyMesh>::Save(m,"out.ply",tri::io::Mask::IOM_FACECOLOR);
  tri::io::ExporterPLY<MyMesh>::Save(profile,"outprof.ply",tri::io::Mask::IOM_FACECOLOR);

  return 0;
}
